@@ -216,16 +216,14 @@ static int geohash_encode_impl(double latitude, double longitude, char* r){
 	char lr[27];
 
 	if(!double_to_i64(latitude/90.0, &lat64) || !double_to_i64(longitude/180.0, &lon64)){
-		return GEOHASH_INVALIDARGUMENT;
+		return GEOHASH_INVALIDARGUMENT; // # nocov
 	}
 	for(int i=0; i<8; i++){
 		interleaved[7-i] = interleave((uint8_t)(lon64>>(i*8)), (uint8_t)(lat64>>(i*8)));
 	}
 
 	int ret = GEOHASH_OK;
-	if((ret=interleaved_to_geohashstr(interleaved, 8, lr, 26)) != GEOHASH_OK){
-		return ret;
-	}
+	if((ret=interleaved_to_geohashstr(interleaved, 8, lr, 26)) != GEOHASH_OK) return ret; // # nocov
 	lr[26] = '\0';
 
 	memcpy(r, (const char*)lr, 27);
@@ -372,8 +370,8 @@ static int geohashstr_to_interleaved(char* r, size_t length, uint16_t *interleav
 	};
 	for(unsigned int i=0; i<length; i++){
 		if(r[i]==0){
-			length = i;
-			break;
+			length = i; // # nocov not hit by intToUtf8(0), rawToChar(as.raw(0)), or ''
+			break; // # nocov
 		}else if(r[i]<0 || map[(unsigned char)r[i]]=='|'){
 			return GEOHASH_INVALIDCODE;
 		}
@@ -437,6 +435,7 @@ static int geohash_decode_impl(char* r, size_t length, double *latitude, double
 	}
 	int ret = GEOHASH_OK;
 	if((ret=geohashstr_to_interleaved(r, length, interleaved, intr_length)) != GEOHASH_OK){
+	  if (intr_free) free(interleaved);
 		return ret;
 	}
 	uint64_t lat64=0;
@@ -542,7 +541,7 @@ List gh_decode_(StringVector geohashes, bool include_delta, int coord_loc) {
       if (ret != GEOHASH_OK) {
         switch (ret) {
         case GEOHASH_NOTSUPPORTED:
-          stop("Unknown endian encountered; please report your use case.");
+          stop("Unknown endian encountered; please report your use case."); // # nocov
         case GEOHASH_INVALIDCODE:
           stop("Invalid geohash; check '%s' at index %d.\nValid characters: [0123456789bcdefghjkmnpqrstuvwxyz]",
                (std::string) geohashes[i], i + 1);
@@ -665,15 +664,15 @@ static int neighbors(uint16_t *interleaved, size_t bitlength, uint16_t* dst, siz
 
 	size_t neighbour_count = 0;
 	for(unsigned int i=0;i<3;i++){
-		if(i>0 && uint8s_cmp(lats[i-1], lats[i], lat_len)==0){
-			continue;
-		}
-		for(int j=0;j<3;j++){
-			if(j>0 && uint8s_cmp(lons[j-1], lons[j], lon_len)==0){
-				continue;
-			}
-			if(i==0 && j==0){
-				continue;
+	  if(i>0 && uint8s_cmp(lats[i-1], lats[i], lat_len)==0){
+	    continue;
+	  }
+	  for(int j=0;j<3;j++){
+	    if(j>0 && uint8s_cmp(lons[j-1], lons[j], lon_len)==0){
+	      continue;
+	    }
+	    if(i==0 && j==0){
+	      continue;
 			}
 
 			for(unsigned int k=0; k<interleaved_length; k++){
@@ -709,8 +708,8 @@ static int geo_neighbors_impl(char *hashcode, char* dst, size_t dst_length, int
 	size_t dst_count = 0;
 	uint16_t *intr_dst = interleaved + interleaved_length;
 	if((ret = neighbors(interleaved, hashcode_length*5, intr_dst, interleaved_length*8, &dst_count) != GEOHASH_OK)){
-		free(interleaved);
-		return ret;
+		free(interleaved); // # nocov alternatives are GEOHASH_INTERNALERROR/GEOHASH_NOMEMORY
+		return ret; // # nocov
 	}
 
 	size_t blen = 0;
@@ -718,15 +717,15 @@ static int geo_neighbors_impl(char *hashcode, char* dst, size_t dst_length, int
 	blen++; // for string NULL terminator
 	char *buffer = (char*)malloc(sizeof(char)*blen);
 	if(buffer==NULL){
-		free(interleaved);
+		free(interleaved); // # nocov
 		return GEOHASH_NOMEMORY; // # nocov
 	}
 
 	for(unsigned int i=0; i<dst_count; i++){
 		if((ret = interleaved_to_geohashstr(intr_dst+i*interleaved_length, interleaved_length, buffer, blen)) != GEOHASH_OK){
-			free(interleaved);
-			free(buffer);
-			return ret;
+			free(interleaved); // # nocov only alternative is GEOHASH_INTERNALERROR
+			free(buffer); // # nocov
+			return ret; // # nocov
 		}
 		buffer[hashcode_length] = '\0';
 		size_t t = hashcode_length + 1;
@@ -766,9 +765,10 @@ List gh_neighbors_(StringVector geohashes, bool self = true) {
     int ret;
     int string_count = 0;
     if((ret = geo_neighbors_impl(geohashes[i], buffer, buffer_sz, &string_count)) != GEOHASH_OK){
+      free(buffer);
       switch (ret) {
       case GEOHASH_NOTSUPPORTED:
-        stop("Unknown endian encountered; please report your use case.");
+        stop("Unknown endian encountered; please report your use case."); // # nocov
       case GEOHASH_INVALIDCODE:
         stop("Invalid geohash; check '%s' at index %d.\nValid characters: [0123456789bcdefghjkmnpqrstuvwxyz]",
              (std::string) geohashes[i], i + 1);
@@ -778,6 +778,7 @@ List gh_neighbors_(StringVector geohashes, bool self = true) {
     }
     // "border" geohashes  should abut the north/south pole
     if (string_count == 5) {
+      // should be able to access [0] because string_count == 5
       switch (geohashes[i][0]) {
       // north pole-adjacent geohashes must start with:
       case 'b':
@@ -811,9 +812,11 @@ List gh_neighbors_(StringVector geohashes, bool self = true) {
         northwest[i] = buffer + 3*blen;
         northeast[i] = buffer + 4*blen;
         break;
-      default:
+      default: {
+        free(buffer); // # nocov
         stop("Internal error. Please report."); // # nocov
       }
+      }
     } else if (string_count == 8) {
       west[i] = buffer;
       east[i] = buffer + blen;
@@ -823,7 +826,15 @@ List gh_neighbors_(StringVector geohashes, bool self = true) {
       north[i] = buffer + 5*blen;
       northwest[i] = buffer + 6*blen;
       northeast[i] = buffer + 7*blen;
-    } else stop("Internal error. Please report."); // # nocov
+    } else if (string_count == 0) {
+      // !nzchar(geohashes[i])
+      west[i] = east[i] = south[i] = southwest[i] = southeast[i] =
+        north[i] = northwest[i] = northeast[i] = NA_STRING;
+    } else {
+      free(buffer); // # nocov
+      stop("Internal error. Please report."); // # nocov
+    }
+    free(buffer);
   }
   if (self) {
     return List::create(
